/**
  *  \file main.cpp
  *  \brief Little Ugly Test Program
  */

#include <iostream>
#include <fstream>
#include <cstdlib>
#include "data.h"
#include "game.h"
#include "util.h"
#include "osdep.h"
#include "program.h"
#include "action.h"
#include "race.h"
#include "expander.h"
#include "selection.h"
#include "loaderstate.h"

static bool
askYesNo(std::string prompt)
{
    std::string reply;
    std::cout << prompt << " (y/n)? ";
    return getline(std::cin, reply) && reply == "y";
}

static void
runAction(Action& a, Selection& sel)
{
    sel.selectAction(&a);
    std::string s = sel.expandString(a.getCommandLine());
    if (s.size() == 0) {
        std::cout << ">> Command not possible\n";
    } else {
        std::cout << "\n>> Running '" << s << "' <<\n\n";
    }
}

class GameDirProvider : public ValueEnumerator<Game> {
    Data& d;
 public:
    GameDirProvider(Data& d)
        : d(d)
        { }
    void handleItem(Game& g) {
        if (g.getLocalDirectory().empty()) {
            std::string n = d.getNewGameDirectory(g.getName(), false);
            if (n.empty()) {
                std::cout << "I cannot allocate a Winplan slot for " << g.getName() << "\n"
                          << "You can assign a directory outside the Winplan tree, which is\n"
                          << "then only accessible to non-Winplan programs.\n"
                          << "Or, leave this directory unassigned, make room by deleting other\n"
                          << "games, and invoke the 'assign' function later to assign this game\n"
                          << "its Winplan slot.\n";
                if (!askYesNo("Assign outside directory"))
                    return;
                n = d.getNewGameDirectory(g.getName(), true);
            }
            g.setLocalDirectory(n);
        }
    }
};

static void
gameMenu(Data& d, Game& g)
{
    std::vector<Program*> p;
    VectorValueEnumerator<Program> pi(p);
    d.enumeratePrograms(pi);

    std::vector<Action*> a;
    VectorValueEnumerator<Action> ai(a);
    d.enumerateSimpleActions(g, ai);

    std::vector<Race*> r;
    VectorValueEnumerator<Race> ri(r);
    g.enumerateRaces(ri);

    Selection sel;
    sel.selectGame(&g);

    std::cout << "-- Game " << g.getName() << " --\n";
    if (sel.getSelectedRace())
        std::cout << "r: race (current is " << sel.getSelectedRace()->getRaceNumber() << ")\n";
    else
        std::cout << "r: race (none selected)\n";
    std::cout << "d: download new results\n";
    std::cout << "u: upload new turn\n";
    std::cout << "w: website\n";
    for (unsigned int i = 0; i < a.size(); ++i)
        std::cout << i << ": " << a[i]->getDescription() << " ("
                  << a[i]->getName() << ", using "
                  << a[i]->getProgram().getName() << ")\n";
    for (unsigned int i = 0; i < p.size(); ++i)
        std::cout << "p" << i << ": " << p[i]->getName() << " ->\n";
    while (1) {
        std::string reply;
        std::cout << "Main Game> ";
        if (!getline(std::cin, reply))
            break;
        int n;
        if (reply == "d") {
            std::cout << "...download...\n";
        } else if (reply == "u") {
            std::cout << "...upload...\n";
        } else if (reply == "w") {
            std::cout << "...website...\n";
        } else if (reply == "q") {
            break;
        } else if (reply == "r") {
            for (unsigned int i = 0; i < r.size(); ++i)
                std::cout << i << ": race " << r[i]->getRaceNumber() << (r[i] == sel.getSelectedRace() ? " <-\n" : "\n");
            if (r.size() == 0)
                std::cout << "No races defined for this game\n";
            std::cout << "Main Game Race> ";
            if (getline(std::cin, reply) && atoi(reply, n) && n >= 0 && n < int(r.size()))
                sel.selectRace(r[n]);
        } else if (atoi(reply, n) && n >= 0 && n < int(a.size())) {
            runAction(*a[n], sel);
        } else if (reply.size() && reply[0] == 'p' && atoi(reply.substr(1), n) && n >= 0 && n < int(p.size())) {
            std::vector<Action*> pa;
            VectorValueEnumerator<Action> pai(pa);
            p[n]->enumerateActions(pai);
            std::cout << "::: Program " << p[n]->getName() << " :::\n";
            for (unsigned int i = 0; i < pa.size(); ++i)
                std::cout << i << ": " << pa[i]->getDescription() << "\n";
            std::cout << "Main Game Program> ";
            if (getline(std::cin, reply) && atoi(reply, n) && n >= 0 && n < int(pa.size())) {
                runAction(*pa[n], sel);
            }
        } else {
            std::cout << "huh?\n";
        }
    }
}

static void
loadFile(Data& d)
{
    std::string s, reply;
    std::cout << "Filename?";
    if (getline(std::cin, s)) {
        std::ifstream in(s.c_str());
        if (!in) {
            std::cerr << "can't open\n";
        } else {
            try {
                Data tmp;
                LoaderState st;
                st.setBaseDirectory(getDirectoryName(s));
                tmp.read(in, st);
                std::cout << "File contains " << st.getNumItems(st.Globals) << " global config definitions,\n"
                          << "  " << st.getNumItems(st.Programs) << " program definitions,\n"
                          << "  " << st.getNumItems(st.Games) << " game definitions.\n";

                st.setAccept(st.Globals,  st.getNumItems(st.Globals)  != 0 && askYesNo("Read globals"));
                st.setAccept(st.Programs, st.getNumItems(st.Programs) != 0 && askYesNo("Read programs"));
                st.setAccept(st.Games,    st.getNumItems(st.Games)    != 0 && askYesNo("Read games"));
                if (st.isAcceptAny()) {
                    in.clear(); // weil wir am Ende waren
                    in.seekg(0);
                    d.read(in, st);

                    GameDirProvider gdp(d);
                    d.enumerateGames(gdp);
                }
            }
            catch (std::exception& e) {
                std::cout << "Load aborted: " << e.what() << "\n";
            }
        }
    }
}

static void
newGame(Data& d)
{
    std::string reply;
    std::string name;
    std::cout << "Game name?";
    if (!getline(std::cin, name))
        return;

    if (d.getGameByName(name)) {
        std::cout << "haben wir schon.\n";
        return;
    }

    std::string dir = d.getNewGameDirectory(name, false);
    if (!dir.size()) {
        std::cout << "You have Winplan, and I cannot find a free game slot.\n";
        if (!askYesNo("Continue and make the game outside the Winplan tree?"))
            return;
        dir = d.getNewGameDirectory(name, true);
    }
    std::cout << "Game will be in '" << dir << "'\n";

    Game* g = new Game(name);
    d.addGame(g);
    g->setLocalDirectory(dir);

    // Noch eine Rasse anlegen
    int nr;
    std::cout << "What race do you play?";
    if (getline(std::cin, reply) && atoi(reply, nr) && nr > 0 && nr <= 11) {
        g->addRace(new Race(nr));
    }
}

static void
mainMenu(Data& d)
{
    std::string reply;
    while (1) {
        std::cout << "=== Game List ===\n";
        std::vector<Game*> g;
        {
            VectorValueEnumerator<Game> vi(g);
            d.enumerateGames(vi);
        }
        std::cout << "load: load definition\n"
                  << "new: new game\n"
                  << "assign: assign missing local dirs\n"
                  << "q: quit\n"
                  << "print: print config file to stdout\n"
                  << "rm: remove game\n";
        for (std::vector<Game*>::size_type i = 0; i < g.size(); ++i)
            std::cout << i << ": " << g[i]->getName() << "\n";
        std::cout << "Main> ";
        if (!getline(std::cin, reply))
            return;
        if (reply == "print") {
            d.write(std::cout);
        } else if (reply == "load") {
            loadFile(d);
        } else if (reply == "q") {
            break;
        } else if (reply == "assign") {
            GameDirProvider gdp(d);
            d.enumerateGames(gdp);
        } else if (reply == "rm") {
            int nr;
            std::cout << "Which one?";
            if (getline(std::cin, reply) && atoi(reply, nr) && nr >= 0 && nr < int(g.size()))
                d.deleteGame(g[nr]);
        } else if (reply == "new") {
            newGame(d);
        } else {
            int num;
            if (atoi(reply, num) && num >= 0 && num < int(g.size()))
                gameMenu(d, *g[num]);
            else
                std::cout << "h?\n";
        }
    }
}

int main(int argc, char** argv)
{
    Data d;
    std::string filename = getSettingsFileName();
    if (argc == 2) {
        filename = argv[1];
    }

    {
        std::ifstream in(filename.c_str());
        if (in) {
            LoaderState st;
            st.setBaseDirectory(getDirectoryName(filename));
            d.read(in, st);
        }
    }
    mainMenu(d);

    {
        std::string bak = filename + ".bak";
        std::remove(bak.c_str());
        std::rename(filename.c_str(), bak.c_str());
        std::ofstream out(filename.c_str());
        if (out) {
            d.write(out);
        }
    }
}
