/**
  *  \file data.h
  *  \brief Integrator Data Management
  */
#ifndef DATA_H_INCLUDED
#define DATA_H_INCLUDED

#include <vector>
#include <istream>
#include <ostream>
#include "propertyobject.h"

class Game;
class Race;
class Program;
class Action;

/** Main data container. */
class Data : public PropertyObject {
    class ReaderState;
    class WriterState;
    friend class ReaderState;
    friend class WriterState;
    
    std::vector<Game*> m_Games;

    std::vector<Program*> m_Programs;

    std::string m_GameBaseDir;

    enum Line { EndOfFile, ClosingBrace, Subtree, Assignment };

    bool readAny(PropertyObject& p, ReaderState& rst, LoaderState& st);
    
    void readUnknown(ReaderState& in);

    void writeGame(Game& g, WriterState& st);

    void writeRace(Race& r, WriterState& st);

    void writeProgram(Program& r, WriterState& st);

    void writeAction(Action& a, WriterState& st);

 public:
    Data();

    ~Data();

    void read(std::istream& in, LoaderState& st);

    void write(std::ostream& out);

    void addGame(Game* g);

    void addProgram(Program* p);

    void enumerate(KVEnumerator& e);

    void enumerateGames(ValueEnumerator<Game>& e);

    void enumeratePrograms(ValueEnumerator<Program>& e);

    void deleteGame(Game* g);

    Game* getGameByName(std::string name);

    Program* getProgramByName(std::string prog);

    Program* getProgramByFeature(std::string feature, bool invert = false);

    bool assign(std::string name, std::string value, LoaderState& st);

    PropertyObject* createSubtree(std::string key, std::string value, LoaderState& st);

    void enumerateSimpleActions(Game& g, ValueEnumerator<Action>& a);

    std::string getNewGameDirectory(std::string gamename, bool force);
};

#endif
