/**
  *  \file expander.cpp
  *  \brief Expander class implementation
  *  \author Stefan Reuther
  */

#include <iostream>
#include "expander.h"

/** Constructor. Initially, no variables are defined. */
Expander::Expander()
{ }

/** Destructor. */
Expander::~Expander()
{ }

void
Expander::handleItem(std::string key, std::string, std::string value)
{
    /* We ignore empty items. In particular, we automatically refuse
       to run utilities for games that don't have a %localdir%. */
    if (value.size())
        m_Variables[key] = value;
}

/** Add all variables from a PropertyObject. */
void
Expander::addObject(PropertyObject& obj)
{
    obj.enumerate(*this);
}

/** Add a single variable. */
void
Expander::addVariable(std::string name, std::string value)
{
    m_Variables[name] = value;
}

/** Expand string. Variable references are specified using \%name%. A single
    percent sign can be specified using \%\%. Non-existant variables cause this
    function to return an empty string.
    \param s string to expand
    \return expanded string. Returns empty string if a non-existant
    variable was referenced */
std::string
Expander::expand(std::string s) const
{
    std::string result;
    while (s.size()) {
        std::string::size_type x = s.find('%');
        if (x == s.npos) {
            result += s;
            break;
        }
        result.append(s, 0, x);
        s.erase(0, x+1);

        x = s.find('%');
        std::string varname;
        if (x == s.npos) {
            varname = s;
            s.erase();
        } else {
            varname.assign(s, 0, x);
            s.erase(0, x+1);
        }
        if (varname.size() == 0) {
            result += "%";
        } else {
            std::map<std::string,std::string>::const_iterator i = m_Variables.find(varname);
            if (i != m_Variables.end())
                result += i->second;
            else
                return std::string();
        }
    }
    return result;
}
