/**
  *  \file util.cpp
  */

#include <string>
#include <stdexcept>
#include <ctype.h>
#include <stdlib.h>
#include "util.h"

/** Trim whitespace. Removes whitespace from both sides.
    \param s [in] Input
    \return s, with leading and trailing whitespace removed. */
std::string
strTrim(std::string s)
{
    return strLTrim(strRTrim(s));
}

/** Trim leading (left) whitespace.
    \param s [in] Input
    \return s, with leading whitespace (see #isspace) removed. */
std::string
strLTrim(std::string s)
{
    std::string::size_type n = 0;
    while (n < s.length() && isspace(s[n]))
        ++n;
    return s.substr(n);
}

/** Trim trailing (right) whitespace.
    \param s [in] Input
    \return s, with trailing whitespace (see #isspace) removed. */
std::string
strRTrim(std::string s)
{
    std::string::size_type n = s.length();
    while (n > 0 && isspace(s[n-1]))
        --n;
    return s.substr(0, n);
}

/** Convert integer to string. Returns the number string. */
std::string
itoa(int i)
{
    char buffer[100];
    std::sprintf(buffer, "%d", i);
    return std::string(buffer);
}

bool
atoi(std::string s, int& i)
{
    const char* p = s.c_str();
    char* endp;
    long l = strtol(p, &endp, 0);
    if (*p != 0 && *endp == 0) {
        i = l;
        return true;
    } else {
        return false;
    }
}

std::string
strLower(std::string s)
{
    for (std::string::size_type i = 0; i < s.size(); ++i)
        s[i] = tolower((unsigned char) s[i]);
    return s;
}

void
fail(std::string what)
{
    throw std::runtime_error(what);
}
